use serde::{Deserialize};
use crate::auth::Session;
use crate::{ApiError, LastfmApiError, LastfmClient};

#[derive(Deserialize, Clone, Debug)]
pub struct GetSessionResponse {
    pub session: Session,
}

pub struct GetSession;

impl GetSession {
    pub async fn send(lastfm_client: &LastfmClient, token: String) -> Result<GetSessionResponse, LastfmApiError> {
        let api_url = lastfm_client.build_get_url("auth.getSession", &[("token", token.as_str())], true);

        let resp = reqwest::get(api_url.as_str())
            .await?
            .text()
            .await?;

        match serde_json::from_str::<GetSessionResponse>(resp.as_str()) {
            Ok(resp) => Ok(resp),
            Err(_) => Err(serde_json::from_str::<ApiError>(resp.as_str())?.into_error())
        }
    }
}