use serde::de::{self, Deserialize, Deserializer, Unexpected};

pub(crate) fn bool_from_string<'de, D>(deserializer: D) -> Result<bool, D::Error>
where
    D: Deserializer<'de>,
{
    match String::deserialize(deserializer)?.as_str() {
        "false" => Ok(false),
        "true" => Ok(true),
        other => Err(de::Error::invalid_value(
            Unexpected::Str(other),
            &"zero or one",
        )),
    }
}

pub(crate) fn bool_from_string_u8<'de, D>(deserializer: D) -> Result<bool, D::Error>
where
    D: Deserializer<'de>,
{
    match String::deserialize(deserializer)?.as_str() {
        "0" => Ok(false),
        "1" => Ok(true),
        other => Err(de::Error::invalid_value(
            Unexpected::Str(other),
            &"zero or one",
        )),
    }
}

pub(crate) fn bool_from_u8<'de, D>(deserializer: D) -> Result<bool, D::Error>
where
    D: Deserializer<'de>,
{
    match u8::deserialize(deserializer)? {
        0 => Ok(false),
        1 => Ok(true),
        other => Err(de::Error::invalid_value(
            Unexpected::Unsigned(other as u64),
            &"zero or one",
        )),
    }
}

pub(crate) fn i32_from_string<'de, D>(deserializer: D) -> Result<i32, D::Error>
where
    D: Deserializer<'de>,
{
    match String::deserialize(deserializer)?.as_str().parse::<i32>() {
        Ok(res) => Ok(res),
        Err(err) => Err(de::Error::invalid_value(Unexpected::Str(&err.to_string()), &"i32")),
    }
}

pub(crate) fn i64_from_string<'de, D>(deserializer: D) -> Result<i64, D::Error>
where
    D: Deserializer<'de>,
{
    match String::deserialize(deserializer)?.as_str().parse::<i64>() {
        Ok(res) => Ok(res),
        Err(err) => Err(de::Error::invalid_value(Unexpected::Str(&err.to_string()), &"i64")),
    }
}